package com.zebra.mobileprintersample;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.Set;
import java.util.StringTokenizer;

import android.app.AlertDialog;
import android.bluetooth.BluetoothDevice;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Handler;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.RadioGroup;
import android.widget.Toast;

import com.zebra.printer.MobilePrinter;

public class DialogManager {

	private static final String[] CODE_PAGE_ITEMS = {
		"Page 0 437 (USA, Standard Europe)",
		"Page 1 Katakana",
		"Page 2 850 (Multilingual)",
		"Page 3 860 (Portuguese)",
		"Page 4 863 (Canadian-French)",
		"Page 5 865 (Nordic)",
		"Page 16 1252 (Latin I)",
		"Page 17 866 (Cyrillic #2)",
		"Page 18 852 (Latin 2)",
		"Page 19 858 (Euro)",
		"Page 21 862 (Hebrew DOS code)",
		"Page 22 864 (Arabic)",
		"Page 23 Thai42",
		"Page 24 1253 (Greek)",
		"Page 25 1254 (Turkish)",
		"Page 26 1257 (Baltic)",
		"Page 27 Farsi",
		"Page 28 1251 (Cyrillic)",
		"Page 29 737 (Greek)",
		"Page 30 775 (Baltic)",
		"Page 31 Thai14",
		"Page 33 1255 (Hebrew New code)",
		"Page 34 Thai 11",
		"Page 35 Thai 18",
		"Page 36 855 (Cyrillic)",
		"Page 37 857 (Turkish)",
		"Page 38 928 (Greek)",
		"Page 39 Thai 16",
		"Page 40 1256 (Arabic)",
		"Page 41 1258 (Vietnam)",
		"Page 42 KHMER(Cambodia)"
	};

	private static final String[] PRINTER_ID_ITEMS = {
		"Firmware version",
		"Manufacturer",
		"Printer model",
		"Code page"
	};
	
	static void showBluetoothDialog(Context context, final Set<BluetoothDevice> pairedDevices) {
		final String[] items = new String[pairedDevices.size()];
		int index = 0;
		for (BluetoothDevice device : pairedDevices) {
			items[index++] = device.getAddress();
		}

		new AlertDialog.Builder(context).setTitle("Paired Bluetooth printers")
				.setItems(items, new DialogInterface.OnClickListener() {
					
					public void onClick(DialogInterface dialog, int which) {
						MainActivity.mMobilePrinter.connect(items[which]);
						
					}
				}).show();
	}

	static void showWifiDialog(AlertDialog dialog, Context context) {
		if (dialog == null) {
			LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
			final View layout = inflater.inflate(R.layout.dialog_wifi, null);

			dialog = new AlertDialog.Builder(context).setView(layout).setTitle("Wi-Fi Connect")
					.setPositiveButton("OK", new DialogInterface.OnClickListener() {

						public void onClick(DialogInterface dialog, int which) {
							EditText editText = (EditText) layout.findViewById(R.id.editText1);
							String ip = editText.getText().toString();

							editText = (EditText) layout.findViewById(R.id.editText2);
							int port = Integer.parseInt(editText.getText().toString());

							MainActivity.mMobilePrinter.connect(ip, port, 5000);
						}
					}).setNegativeButton("Cancel", new DialogInterface.OnClickListener() {

						public void onClick(DialogInterface dialog, int which) {
							// TODO Auto-generated method stub

						}
					}).create();
		}
		dialog.show();
	}

	static void showPdf417Dialog(AlertDialog dialog, final Context context) {
		if (dialog == null) {
			LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
			final View layout = inflater.inflate(R.layout.dialog_print_pdf417, null);

			dialog = new AlertDialog.Builder(context).setView(layout).setTitle("PDF417")
					.setPositiveButton("OK", new DialogInterface.OnClickListener() {

						public void onClick(DialogInterface dialog, int which) {
							EditText editText = (EditText) layout.findViewById(R.id.editText1);
							String data = editText.getText().toString();

							editText = (EditText) layout.findViewById(R.id.editText2);
							String string = editText.getText().toString();
							if (string.length() == 0) {
								Toast.makeText(context, "Please enter the width again.", Toast.LENGTH_SHORT).show();
								return;
							}
							int width = Integer.parseInt(string);

							editText = (EditText) layout.findViewById(R.id.editText3);
							string = editText.getText().toString();
							if (string.length() == 0) {
								Toast.makeText(context, "Please enter the height again.", Toast.LENGTH_SHORT).show();
								return;
							}
							int height = Integer.parseInt(string);
							
							int alignment = MobilePrinter.ALIGNMENT_LEFT;
							RadioGroup radioGroup = (RadioGroup) layout.findViewById(R.id.radioGroup1);
							switch (radioGroup.getCheckedRadioButtonId()) {
							case R.id.radio1:
								alignment = MobilePrinter.ALIGNMENT_CENTER;
								break;
								
							case R.id.radio2:
								alignment = MobilePrinter.ALIGNMENT_RIGHT;
								break;
							}

							CheckBox checkBox = (CheckBox) layout.findViewById(R.id.checkBox1);
							if (checkBox.isChecked()) {
								MainActivity.mMobilePrinter.printPdf417(data, alignment, width, height, false);
								MainActivity.mMobilePrinter.formFeed(true);
							} else {
								MainActivity.mMobilePrinter.printPdf417(data, alignment, width, height, true);
							}

						}
					}).setNegativeButton("Cancel", new DialogInterface.OnClickListener() {

						public void onClick(DialogInterface dialog, int which) {
							// TODO Auto-generated method stub

						}
					}).create();
		}
		dialog.show();
	}

	static void showQrCodeDialog(AlertDialog dialog, final Context context) {
		if (dialog == null) {
			LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
			final View layout = inflater.inflate(R.layout.dialog_print_qrcode, null);

			dialog = new AlertDialog.Builder(context).setView(layout).setTitle("QR Code")
					.setPositiveButton("OK", new DialogInterface.OnClickListener() {

						public void onClick(DialogInterface dialog, int which) {
							EditText editText = (EditText) layout.findViewById(R.id.editText1);
							String data = editText.getText().toString();

							int model = MobilePrinter.QR_CODE_MODEL2;
							RadioGroup radioGroup = (RadioGroup) layout.findViewById(R.id.radioGroup1);
							if (radioGroup.getCheckedRadioButtonId() == R.id.radio0) {
								model = MobilePrinter.QR_CODE_MODEL1;
							}

							editText = (EditText) layout.findViewById(R.id.editText2);
							String string = editText.getText().toString();
							if (string.length() == 0) {
								Toast.makeText(context, "Please enter the size again.", Toast.LENGTH_SHORT).show();
								return;
							}
							int size = Integer.parseInt(string);
							
							int alignment = MobilePrinter.ALIGNMENT_LEFT;
							radioGroup = (RadioGroup) layout.findViewById(R.id.radioGroup2);
							switch (radioGroup.getCheckedRadioButtonId()) {
							case R.id.radio3:
								alignment = MobilePrinter.ALIGNMENT_CENTER;
								break;
								
							case R.id.radio4:
								alignment = MobilePrinter.ALIGNMENT_RIGHT;
								break;
							}

							CheckBox checkBox = (CheckBox) layout.findViewById(R.id.checkBox1);
							if (checkBox.isChecked()) {
								MainActivity.mMobilePrinter.printQrCode(data, alignment, model, size, false);
								MainActivity.mMobilePrinter.formFeed(true);
							} else {
								MainActivity.mMobilePrinter.printQrCode(data, alignment, model, size, true);
							}
						}
					}).setNegativeButton("Cancel", new DialogInterface.OnClickListener() {

						public void onClick(DialogInterface dialog, int which) {
							// TODO Auto-generated method stub

						}
					}).create();
		}
		dialog.show();
	}

	static void showMaxiCodeDialog(AlertDialog dialog, Context context) {
		if (dialog == null) {
			LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
			final View layout = inflater.inflate(R.layout.dialog_print_maxicode, null);

			dialog = new AlertDialog.Builder(context).setView(layout).setTitle("Maxi Code")
					.setPositiveButton("OK", new DialogInterface.OnClickListener() {

						public void onClick(DialogInterface dialog, int which) {
							EditText editText = (EditText) layout.findViewById(R.id.editText1);
							String data = editText.getText().toString();

							int mode = MobilePrinter.MAXI_CODE_MODE2;
							RadioGroup radioGroup = (RadioGroup) layout.findViewById(R.id.radioGroup1);
							switch (radioGroup.getCheckedRadioButtonId()) {
							case R.id.radio0:
								mode = MobilePrinter.MAXI_CODE_MODE2;
								byte[] header = new byte[] {
										'[', ')', '>', 0x1e, '0', '1', 0x1d, '9', '6', '1',
										'2', '3', '4', '5', '6', '7', '8', '9', 0x1d, '0',
										'0', '7', 0x1d, '2', '5', '0', 0x1d};
								data = new String(header) + data;
								break;

							case R.id.radio1:
								mode = MobilePrinter.MAXI_CODE_MODE3;
								header = new byte[] {
										'[', ')', '>', 0x1e, '0', '1', 0x1d, '9', '6', 'F',
										'A', 'B', 'C', 'D', 'E', 0x1d, '0', '0', '7', 0x1d,
										'2', '5', '0', 0x1d};
								data = new String(header) + data;
								break;

							case R.id.radio2:
								mode = MobilePrinter.MAXI_CODE_MODE4;
								header = new byte[] {'E', 0x1d, '7', 0x1d, '5', 0x1d};
								data = new String(header) + data;
								break;
							}
							
							int alignment = MobilePrinter.ALIGNMENT_LEFT;
							radioGroup = (RadioGroup) layout.findViewById(R.id.radioGroup2);
							switch (radioGroup.getCheckedRadioButtonId()) {
							case R.id.radio4:
								alignment = MobilePrinter.ALIGNMENT_CENTER;
								break;
								
							case R.id.radio5:
								alignment = MobilePrinter.ALIGNMENT_RIGHT;
								break;
							}

							MainActivity.mMobilePrinter.printMaxiCode(data, alignment, mode, true);
							
							CheckBox checkBox = (CheckBox) layout.findViewById(R.id.checkBox1);
							if (checkBox.isChecked()) {
								MainActivity.mMobilePrinter.formFeed(false);
							}
						}
					}).setNegativeButton("Cancel", new DialogInterface.OnClickListener() {

						public void onClick(DialogInterface dialog, int which) {
							// TODO Auto-generated method stub

						}
					}).create();
		}
		dialog.show();
	}

	static void showDataMatrixDialog(AlertDialog dialog, Context context) {
		if (dialog == null) {
			LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
			final View layout = inflater.inflate(R.layout.dialog_print_datamatrix, null);

			dialog = new AlertDialog.Builder(context).setView(layout).setTitle("DATAMATRIX")
					.setPositiveButton("OK", new DialogInterface.OnClickListener() {

						public void onClick(DialogInterface dialog, int which) {
							EditText editText = (EditText) layout.findViewById(R.id.editText1);
							String data = editText.getText().toString();

							editText = (EditText) layout.findViewById(R.id.editText2);
							int size = Integer.parseInt(editText.getText().toString());
							
							int alignment = MobilePrinter.ALIGNMENT_LEFT;
							RadioGroup radioGroup = (RadioGroup) layout.findViewById(R.id.radioGroup1);
							switch (radioGroup.getCheckedRadioButtonId()) {
							case R.id.radio1:
								alignment = MobilePrinter.ALIGNMENT_CENTER;
								break;
								
							case R.id.radio2:
								alignment = MobilePrinter.ALIGNMENT_RIGHT;
								break;
							}

							MainActivity.mMobilePrinter.printDataMatrix(data, alignment, size, true);
							
							CheckBox checkBox = (CheckBox) layout.findViewById(R.id.checkBox1);
							if (checkBox.isChecked()) {
								MainActivity.mMobilePrinter.formFeed(false);
							}
						}
					}).setNegativeButton("Cancel", new DialogInterface.OnClickListener() {

						public void onClick(DialogInterface dialog, int which) {
							// TODO Auto-generated method stub

						}
					}).create();
		}
		dialog.show();
	}

	static void showCodePageDialog(AlertDialog dialog, Context context, final Handler handler) {
		if (dialog == null) {
			dialog = new AlertDialog.Builder(context).setTitle("Code page")
					.setItems(CODE_PAGE_ITEMS, new DialogInterface.OnClickListener() {

						public void onClick(DialogInterface dialog, int which) {
							switch (which) {
							case 0:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_437_USA);
								break;
							case 1:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_KATAKANA);
								break;
							case 2:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_850_MULTILINGUAL);
								break;
							case 3:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_860_PORTUGUESE);
								break;
							case 4:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_863_CANADIAN_FRENCH);
								break;
							case 5:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_865_NORDIC);
								break;
							case 6:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_1252_LATIN1);
								break;
							case 7:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_866_CYRILLIC2);
								break;
							case 8:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_852_LATIN2);
								break;
							case 9:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_858_EURO);
								break;
							case 10:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_862_HEBREW_DOS_CODE);
								break;
							case 11:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_864_ARABIC);
								break;
							case 12:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_THAI42);
								break;
							case 13:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_1253_GREEK);
								break;
							case 14:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_1254_TURKISH);
								break;
							case 15:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_1257_BALTIC);
								break;
							case 16:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_FARSI);
								break;
							case 17:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_1251_CYRILLIC);
								break;
							case 18:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_737_GREEK);
								break;
							case 19:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_775_BALTIC);
								break;
							case 20:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_437_THAI14);
								break;
							case 21:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_1255_HEBREW_NEW_CODE);
								break;
							case 22:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_THAI11);
								break;
							case 23:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_THAI18);
								break;
							case 24:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_855_CYRILLIC);
								break;
							case 25:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_857_TURKISH);
								break;
							case 26:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_928_GREEK);
								break;
							case 27:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_THAI16);
								break;
							case 28:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_1256_ARABIC);
								break;
							case 29:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_1258_VIETNAM);
								break;
							case 30:
								MainActivity.mMobilePrinter.setCodePage(MobilePrinter.CODE_PAGE_KHMER_CAMBODIA);
								break;
							}

						}
					}).create();
		}
		dialog.show();
	}

	static void showPrinterIdDialog(AlertDialog dialog, Context context) {
		if (dialog == null) {
			dialog = new AlertDialog.Builder(context).setTitle("Get printer ID")
					.setItems(PRINTER_ID_ITEMS, new DialogInterface.OnClickListener() {

						public void onClick(DialogInterface dialog, int which) {
							switch (which) {
							case 0:
								MainActivity.mMobilePrinter.getPrinterId(MobilePrinter.PRINTER_ID_FIRMWARE_VERSION);
								break;
							case 1:
								MainActivity.mMobilePrinter.getPrinterId(MobilePrinter.PRINTER_ID_MANUFACTURER);
								break;
							case 2:
								MainActivity.mMobilePrinter.getPrinterId(MobilePrinter.PRINTER_ID_PRINTER_MODEL);
								break;
							case 3:
								MainActivity.mMobilePrinter.getPrinterId(MobilePrinter.PRINTER_ID_CODE_PAGE);
								break;
							}

						}
					}).create();

		}
		dialog.show();
	}

	static void showDirectIoDialog(AlertDialog dialog, final Context context) {
		if (dialog == null) {
			LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
			final View view = inflater.inflate(R.layout.dialog_direct_io, null);

			dialog = new AlertDialog.Builder(context).setView(view).setTitle("Direct IO")
					.setPositiveButton("OK", new DialogInterface.OnClickListener() {

						public void onClick(DialogInterface dialog, int which) {
							EditText editText = (EditText) view.findViewById(R.id.editText1);
							String string = editText.getText().toString();
							StringTokenizer stringTokenize = new StringTokenizer(string);

							ArrayList<Byte> arrayList = new ArrayList<Byte>();
							while (stringTokenize.hasMoreTokens()) {
								try {
									byte b = Byte.parseByte(stringTokenize.nextToken(), 16);
									arrayList.add(b);
								} catch (NumberFormatException e) {
									e.printStackTrace();
									Toast.makeText(context, "Invalid command!", Toast.LENGTH_SHORT).show();
									return;
								}
							}

							CheckBox checkBox = (CheckBox) view.findViewById(R.id.checkBox1);
							boolean hasResponse = checkBox.isChecked();

							if (arrayList.size() > 0) {
								ByteBuffer buffer = ByteBuffer.allocate(arrayList.size());
								for (int i = 0; i < arrayList.size(); i++) {
									buffer.put(arrayList.get(i));
								}

								MainActivity.mMobilePrinter.executeDirectIo(buffer.array(), hasResponse);
							}

						}
					}).setNegativeButton("Cancel", new DialogInterface.OnClickListener() {

						public void onClick(DialogInterface dialog, int which) {
							// TODO Auto-generated method stub

						}
					}).create();
		}
		dialog.show();
	}

	static void showPageModeDialog(AlertDialog dialog, final Context context) {
		if (dialog == null) {
			LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
			final View view = inflater.inflate(R.layout.dialog_page_mode, null);
			dialog = new AlertDialog.Builder(context).setView(view).setTitle("Set page mode")
					.setPositiveButton("OK", new DialogInterface.OnClickListener() {

						public void onClick(DialogInterface dialog, int which) {
							RadioGroup radioGroup = (RadioGroup) view.findViewById(R.id.radioGroup1);
							if (radioGroup.getCheckedRadioButtonId() == R.id.radio0) {
								MainActivity.mMobilePrinter.setStandardMode();
							} else {
								MainActivity.mMobilePrinter.setPageMode();
								
								EditText editText = (EditText) view.findViewById(R.id.editText1);
								String string = editText.getText().toString();
								if (string.length() == 0) {
									Toast.makeText(context, "Please enter the horizontal start position again", Toast.LENGTH_SHORT).show();
									return;
								}
								int x = Integer.parseInt(string);
								
								editText = (EditText) view.findViewById(R.id.editText2);
								string = editText.getText().toString();
								if (string.length() == 0) {
									Toast.makeText(context, "Please enter the vertical start position again", Toast.LENGTH_SHORT).show();
									return;
								}
								int y = Integer.parseInt(string);
								
								editText = (EditText) view.findViewById(R.id.editText3);
								string = editText.getText().toString();
								if (string.length() == 0) {
									Toast.makeText(context, "Please enter the horizontal print area again", Toast.LENGTH_SHORT).show();
									return;
								}
								int width = Integer.parseInt(string);
								
								editText = (EditText) view.findViewById(R.id.editText4);
								string = editText.getText().toString();
								if (string.length() == 0) {
									Toast.makeText(context, "Please enter the vertical print area again", Toast.LENGTH_SHORT).show();
									return;
								}
								int height = Integer.parseInt(string);
								
								MainActivity.mMobilePrinter.setPrintArea(x, y, width, height);
								
								radioGroup = (RadioGroup) view.findViewById(R.id.radioGroup2);
								switch (radioGroup.getCheckedRadioButtonId()) {
								case R.id.radio2:
									MainActivity.mMobilePrinter.setPrintDirection(MobilePrinter.DIRECTION_LEFT_TO_RIGHT);
									break;
								case R.id.radio3:
									MainActivity.mMobilePrinter.setPrintDirection(MobilePrinter.DIRECTION_BOTTOM_TO_TOP);
									break;
								case R.id.radio4:
									MainActivity.mMobilePrinter.setPrintDirection(MobilePrinter.DIRECTION_RIGHT_TO_LEFT);
									break;
								case R.id.radio5:
									MainActivity.mMobilePrinter.setPrintDirection(MobilePrinter.DIRECTION_TOP_TO_BOTTOM);
									break;
								}
								
								editText = (EditText) view.findViewById(R.id.editText5);
								string = editText.getText().toString();
								if (string.length() > 0) {
									int position = Integer.parseInt(string);
									MainActivity.mMobilePrinter.setAbsolutePrintPosition(position);
								}
								
								editText = (EditText) view.findViewById(R.id.editText6);
								string = editText.getText().toString();
								if (string.length() > 0) {
									int position = Integer.parseInt(string);
									MainActivity.mMobilePrinter.setAbsoluteVerticalPrintPosition(position);
								}
							}

						}
					}).setNegativeButton("Cancel", new DialogInterface.OnClickListener() {

						public void onClick(DialogInterface dialog, int which) {
							// TODO Auto-generated method stub

						}
					}).create();
		}
		dialog.show();
	}
}
