package com.zebra.mobileprintersample;

import java.util.ArrayList;
import java.util.Set;

import android.annotation.SuppressLint;
import android.app.ActionBar;
import android.app.AlertDialog;
import android.app.ListActivity;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.Toast;

import com.zebra.printer.MobilePrinter;

@SuppressLint("HandlerLeak")
public class MainActivity extends ListActivity {
	
	static final String ACTION_GET_DEFINEED_NV_IMAGE_KEY_CODES = "com.zebra.anction.GET_DEFINED_NV_IMAGE_KEY_CODES";
	static final String ACTION_GET_MSR_TRACK_DATA = "com.zebra.anction.GET_MSR_TRACK_DATA";
	static final String ACTION_GET_MSR_ENCRYPTION = "com.zebra.anction.GET_MSR_ENCRYPTION";
	
	static final String EXTRA_NAME_NV_KEY_CODES = "NvKeyCodes";
	static final String EXTRA_NAME_MSR_MODE = "MsrMode";
	static final String EXTRA_NAME_MSR_TRACK_DATA = "MsrTrackData";
	static final String EXTRA_NAME_MSR_ENCRYPTION = "MsrEncryption";
	
	static final int REQUEST_CODE_SELECT_FIRMWARE = Integer.MAX_VALUE;
	static final int RESULT_CODE_SELECT_FIRMWARE = Integer.MAX_VALUE - 1;
	static final int REQUEST_CODE_MSR_MANAGER = Integer.MAX_VALUE - 2;
	static final int RESULT_CODE_MSR_MANAGER = Integer.MAX_VALUE - 3;
	
	static final int MESSAGE_START_WORK = Integer.MAX_VALUE - 4;
	static final int MESSAGE_END_WORK = Integer.MAX_VALUE - 5;
	
	static final String FIRMWARE_FILE_NAME = "FirmwareFileName";

	// Name of the connected device
	private String mConnectedDeviceName = null;
	
	private ListView mListView;
	private ProgressBar mProgressBar;
	
	private AlertDialog mWifiDialog;
	private AlertDialog mPdf417Dialog;
	private AlertDialog mQrCodeDialog;
	private AlertDialog mMaxiCodeDialog;
	private AlertDialog mDataMatrixDialog;
	private AlertDialog mCodePageDialog;
	private AlertDialog mPrinterIdDialog;
	private AlertDialog mPageModeDialog;
	private AlertDialog mDirectIoDialog;
	
	static MobilePrinter mMobilePrinter;
	
	private static final String[] FUNCTIONS = {
		// Status check
		"getStatus",				// 0
		"getBatteryStatus",			// 1
		"enable automatic status back",	// 2
		"disable automatic status back",// 3
		
		// Configuration
		"getPrinterId",				// 4
		"setCodePage",				// 5
		"set page mode",			// 6
		"initialize",				// 7
		"setLabelMode",				// 8
		"executeAutomaticCalibration",	// 9
		"setReceiptMode",			// 10
		
		// Print
		"printText",				// 11
		"print1dBarcode",			// 12
		"printPdf417",				// 13
		"printQrCode",				// 14
		"printMaxiCode",			// 15
		"printDataMatrix",			// 16
		"printBitmap",				// 17
		
		// Additional functions
		"executeDirectIo",			// 18
		"NV image manager",			// 29
		"MSR manager",				// 20
		"updateFirmware"			// 21
	};
	
	private boolean mIsConnected;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);

		ArrayList<String> list = new ArrayList<String>();
		for (int i = 0; i < FUNCTIONS.length; i++) {
			list.add(FUNCTIONS[i]);
		}

		ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, list);
		mListView = (ListView) findViewById(android.R.id.list);
		mListView.setAdapter(adapter);
		mListView.setEnabled(false);
		
		mProgressBar = (ProgressBar) findViewById(R.id.progressBar1);
		
		mMobilePrinter = new MobilePrinter(this, mHandler, null);
	}

	@Override
	public void onDestroy() {
		try {
			unregisterReceiver(mUsbReceiver);
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		}
		mMobilePrinter.disconnect();
		super.onDestroy();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.activity_main, menu);
		return true;
	}
	
	@Override
	public boolean onPrepareOptionsMenu(Menu menu) {
		if (mIsConnected) {
			menu.getItem(0).setEnabled(false);
			menu.getItem(1).setEnabled(false);
			menu.getItem(2).setEnabled(false);
			menu.getItem(3).setEnabled(true);
		} else {
			menu.getItem(0).setEnabled(true);
			menu.getItem(1).setEnabled(true);
			menu.getItem(2).setEnabled(true);
			menu.getItem(3).setEnabled(false);
		}
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.item1:
			mMobilePrinter.findBluetoothPrinters();
			break;

		case R.id.item2:
			DialogManager.showWifiDialog(mWifiDialog, MainActivity.this);
			return true;

		case R.id.item3:
			mMobilePrinter.connect();

			// listen for new devices
			IntentFilter filter = new IntentFilter();
			filter.addAction(UsbManager.ACTION_USB_DEVICE_ATTACHED);
			filter.addAction(UsbManager.ACTION_USB_DEVICE_DETACHED);
			registerReceiver(mUsbReceiver, filter);
			return true;

		case R.id.item4:
			mMobilePrinter.disconnect();
			return true;
		}
		return false;
	}

	@Override
	protected void onListItemClick(ListView l, View v, int position, long id) {
		switch (position) {
		case 0:	// getStatus
			mMobilePrinter.getStatus();
			break;
			
		case 1: // getBatteryStatus
			mMobilePrinter.getBatteryStatus();
			break;
			
		case 2: // enable automatic status back
			mMobilePrinter.automateStatusBack(true);
			break;
			
		case 3: // disable automatic status back
			mMobilePrinter.automateStatusBack(false);
			break;
			
		case 4: // getPrinterId
			DialogManager.showPrinterIdDialog(mPrinterIdDialog, MainActivity.this);
			break;
			
		case 5: // setCodePage
			DialogManager.showCodePageDialog(mCodePageDialog, MainActivity.this, mHandler);
			break;		
			
		case 6: // set page mode
			DialogManager.showPageModeDialog(mPageModeDialog, MainActivity.this);
			break;
			
		case 7:	// initialize
			mMobilePrinter.initialize();
			break;
			
		case 8:	// setLabelMode
			mMobilePrinter.setLabelMode();
			break;
			
		case 9:	// executeAutomaticCalibration
			mMobilePrinter.executeAutomaticCalibration();
			break;
			
		case 10:	// setReceiptMode
			mMobilePrinter.setReceiptMode();
			break;
			
		case 11:	// printText
			Intent intent = new Intent(MainActivity.this, PrintTextActivity.class);
			startActivity(intent);
			break;

		case 12:	// print1dBarcode
			intent = new Intent(MainActivity.this, Print1dBarcodeActivity.class);
			startActivity(intent);
			break;

		case 13:	// printPdf417
			DialogManager.showPdf417Dialog(mPdf417Dialog, MainActivity.this);
			break;

		case 14:	// printQrCode
			DialogManager.showQrCodeDialog(mQrCodeDialog, MainActivity.this);
			break;

		case 15:	// printMaxiCode
			DialogManager.showMaxiCodeDialog(mMaxiCodeDialog, MainActivity.this);
			break;

		case 16: // printDataMatrix
			DialogManager.showDataMatrixDialog(mDataMatrixDialog, MainActivity.this);
			break;

		case 17: // printBitmap
			intent = new Intent(MainActivity.this, PrintBitmapAcitivity.class);
			startActivity(intent);
			break;		
			
		case 18: // directIo
			DialogManager.showDirectIoDialog(mDirectIoDialog, MainActivity.this);
			break;
			
		case 19: // NV image manager
			intent = new Intent(MainActivity.this, NvImageActivity.class);
			startActivity(intent);
			break;
			
		case 20: // MSR manager
			mMobilePrinter.getMsrMode();
			break;
			
		case 21: // update firmware
			intent = new Intent(MainActivity.this, FileExplorerActivity.class);
			startActivityForResult(intent, REQUEST_CODE_SELECT_FIRMWARE);
			break;
		}
	}
	
	@Override
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		IntentFilter filter = new IntentFilter();
		filter.addAction(UsbManager.ACTION_USB_DEVICE_ATTACHED);
		filter.addAction(UsbManager.ACTION_USB_DEVICE_DETACHED);
		registerReceiver(mUsbReceiver, filter);
		
		if (requestCode == REQUEST_CODE_SELECT_FIRMWARE && resultCode == RESULT_CODE_SELECT_FIRMWARE) {
			final String binaryFilePath = data.getStringExtra(FIRMWARE_FILE_NAME);
			mHandler.obtainMessage(MESSAGE_START_WORK).sendToTarget();
			new Thread(new Runnable() {
				
				public void run() {
					mMobilePrinter.updateFirmware(binaryFilePath);
					try {
						Thread.sleep(5000);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					mHandler.obtainMessage(MESSAGE_END_WORK).sendToTarget();
				}
			}).start();
		} else if (requestCode == REQUEST_CODE_MSR_MANAGER && resultCode == RESULT_CODE_MSR_MANAGER) {
			Toast.makeText(getApplicationContext(), "Please reset the printer", Toast.LENGTH_SHORT).show();
			new Thread(new Runnable() {
				
				@Override
				public void run() {
					try {
						Thread.sleep(1000);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					mMobilePrinter.disconnect();
				}
			}).start();
		} else {
			super.onActivityResult(requestCode, resultCode, data);
		}
	}

	private final void setStatus(int resId) {
		final ActionBar actionBar = getActionBar();
		actionBar.setSubtitle(resId);
	}

	private final void setStatus(CharSequence subtitle) {
		final ActionBar actionBar = getActionBar();
		actionBar.setSubtitle(subtitle);
	}

	private void dispatchMessage(Message msg) {
		switch (msg.arg1) {
		case MobilePrinter.PROCESS_GET_STATUS:
			if (msg.arg2 == MobilePrinter.STATUS_NORMAL) {
				Toast.makeText(getApplicationContext(), "No error", Toast.LENGTH_SHORT).show();
			} else {
				StringBuffer buffer = new StringBuffer();
				if ((msg.arg2 & MobilePrinter.STATUS_COVER_OPEN) == MobilePrinter.STATUS_COVER_OPEN) {
					buffer.append("Cover is open.\n");
				}
				if ((msg.arg2 & MobilePrinter.STATUS_PAPER_NOT_PRESENT) == MobilePrinter.STATUS_PAPER_NOT_PRESENT) {
					buffer.append("Paper end sensor: paper not present.\n");
				}

				Toast.makeText(getApplicationContext(), buffer.toString(), Toast.LENGTH_SHORT).show();
			}
			break;
			
		case MobilePrinter.PROCESS_GET_BATTERY_STATUS:
			switch (msg.arg2) {
			case MobilePrinter.STATUS_BATTERY_FULL:
				Toast.makeText(getApplicationContext(), "Full voltage", Toast.LENGTH_SHORT).show();
				break;
				
			case MobilePrinter.STATUS_BATTERY_HIGH:
				Toast.makeText(getApplicationContext(), "High voltage", Toast.LENGTH_SHORT).show();
				break;
				
			case MobilePrinter.STATUS_BATTERY_MIDDLE:
				Toast.makeText(getApplicationContext(), "Middle voltage", Toast.LENGTH_SHORT).show();
				break;
				
			case MobilePrinter.STATUS_BATTERY_LOW:
				Toast.makeText(getApplicationContext(), "Low voltage", Toast.LENGTH_SHORT).show();
				break;
			}
			break;
			
		case MobilePrinter.PROCESS_GET_PRINTER_ID:
			Toast.makeText(getApplicationContext(), (String) msg.obj, Toast.LENGTH_SHORT).show();
			break;
			
		case MobilePrinter.PROCESS_AUTO_STATUS_BACK:
			StringBuffer buffer = new StringBuffer(0);
			if ((msg.arg2 & MobilePrinter.AUTO_STATUS_COVER_OPEN) == MobilePrinter.AUTO_STATUS_COVER_OPEN) {
				buffer.append("Cover is open.\n");
			}
			if ((msg.arg2 & MobilePrinter.AUTO_STATUS_NO_PAPER) == MobilePrinter.AUTO_STATUS_NO_PAPER) {
				buffer.append("Paper end sensor: no paper present.\n");
			}
			
			if (buffer.capacity() > 0) {
				Toast.makeText(getApplicationContext(), buffer.toString(), Toast.LENGTH_SHORT).show();
			} else {
				Toast.makeText(getApplicationContext(), "No error.", Toast.LENGTH_SHORT).show();
			}
			break;
			
		case MobilePrinter.PROCESS_GET_NV_IMAGE_KEY_CODES:
			Bundle bundle = msg.getData();
			int[] value = bundle.getIntArray(MobilePrinter.NV_IMAGE_KEY_CODES);
			
			Intent intent = new Intent();
			intent.setAction(ACTION_GET_DEFINEED_NV_IMAGE_KEY_CODES);
			intent.putExtra(EXTRA_NAME_NV_KEY_CODES, value);
			sendBroadcast(intent);
			break;
			
		case MobilePrinter.PROCESS_EXECUTE_DIRECT_IO:
			buffer = new StringBuffer();
			byte[] response = (byte[]) msg.obj;
			for (int i = 0; i < response.length && response[i] != 0; i++) {
				buffer.append(Integer.toHexString(response[i]) + " ");
			}
			
			Toast.makeText(getApplicationContext(), buffer.toString(), Toast.LENGTH_SHORT).show();
			break;
			
		case MobilePrinter.PROCESS_MSR_TRACK:
			intent = new Intent();
			intent.setAction(ACTION_GET_MSR_TRACK_DATA);
			intent.putExtra(EXTRA_NAME_MSR_TRACK_DATA, msg.getData());
			sendBroadcast(intent);
			break;
			
		case MobilePrinter.PROCESS_GET_MSR_MODE:
			intent = new Intent(MainActivity.this, MsrActivity.class);
			intent.putExtra(EXTRA_NAME_MSR_MODE, msg.arg2);
			startActivityForResult(intent, REQUEST_CODE_MSR_MANAGER);
			break;
			
		case MobilePrinter.PROCESS_GET_MSR_ENCRYPTION:
			intent = new Intent();
			intent.setAction(ACTION_GET_MSR_ENCRYPTION);
			intent.putExtra(EXTRA_NAME_MSR_ENCRYPTION, msg.arg2);
			sendBroadcast(intent);
			break;
		}
	}

	private final Handler mHandler = new Handler() {
		@SuppressWarnings("unchecked")
		@Override
		public void handleMessage(Message msg) {
			switch (msg.what) {
			case MobilePrinter.MESSAGE_STATE_CHANGE:
				switch (msg.arg1) {
				case MobilePrinter.STATE_CONNECTED:
					setStatus(getString(R.string.title_connected_to, mConnectedDeviceName));
					mListView.setEnabled(true);
					mIsConnected = true;
					invalidateOptionsMenu();
					break;

				case MobilePrinter.STATE_CONNECTING:
					setStatus(R.string.title_connecting);
					break;

				case MobilePrinter.STATE_NONE:
					setStatus(R.string.title_not_connected);
					mListView.setEnabled(false);
					mIsConnected = false;
					invalidateOptionsMenu();
					mProgressBar.setVisibility(View.INVISIBLE);
					break;
				}
				break;
				
			case MobilePrinter.MESSAGE_WRITE:
				switch (msg.arg1) {
				case MobilePrinter.PROCESS_DEFINE_NV_IMAGE:
					mMobilePrinter.getDefinedNvImageKeyCodes();
					Toast.makeText(getApplicationContext(), "Complete to define NV image", Toast.LENGTH_LONG).show();
					break;
					
				case MobilePrinter.PROCESS_REMOVE_NV_IMAGE:
					mMobilePrinter.getDefinedNvImageKeyCodes();
					Toast.makeText(getApplicationContext(), "Complete to remove NV image", Toast.LENGTH_LONG).show();
					break;
					
				case MobilePrinter.PROCESS_UPDATE_FIRMWARE:
					mMobilePrinter.disconnect();
					Toast.makeText(getApplicationContext(), "Complete to download firmware.\nPlease reboot the printer.", Toast.LENGTH_SHORT).show();
					break;
				}
				break;

			case MobilePrinter.MESSAGE_READ:
				MainActivity.this.dispatchMessage(msg);
				break;

			case MobilePrinter.MESSAGE_DEVICE_NAME:
				mConnectedDeviceName = msg.getData().getString(MobilePrinter.DEVICE_NAME);
				Toast.makeText(getApplicationContext(), mConnectedDeviceName, Toast.LENGTH_LONG).show();
				break;

			case MobilePrinter.MESSAGE_TOAST:
				mListView.setEnabled(false);
				Toast.makeText(getApplicationContext(), msg.getData().getString(MobilePrinter.TOAST), Toast.LENGTH_SHORT).show();
				break;
				
			case MobilePrinter.MESSAGE_DEVICE_SET:
				if (msg.obj == null) {
					Toast.makeText(getApplicationContext(), "No paired device", Toast.LENGTH_SHORT).show();
				} else {
					DialogManager.showBluetoothDialog(MainActivity.this, (Set<BluetoothDevice>) msg.obj);
				}
				break;
				
			case MobilePrinter.MESSAGE_PRINT_COMPLETE:
				Toast.makeText(getApplicationContext(), "Complete to print", Toast.LENGTH_SHORT).show();
				break;
				
			case MESSAGE_START_WORK:
				mListView.setEnabled(false);
				mProgressBar.setVisibility(View.VISIBLE);
				break;
				
			case MESSAGE_END_WORK:
				mListView.setEnabled(true);
				mProgressBar.setVisibility(View.INVISIBLE);
				break;
			}
		}
	};

	private BroadcastReceiver mUsbReceiver = new BroadcastReceiver() {

		@Override
		public void onReceive(Context context, Intent intent) {
			String action = intent.getAction();

			if (UsbManager.ACTION_USB_DEVICE_ATTACHED.equals(action)) {
				mMobilePrinter.connect();
				Toast.makeText(getApplicationContext(), "Found USB device", Toast.LENGTH_SHORT).show();
			} else if (UsbManager.ACTION_USB_DEVICE_DETACHED.equals(action)) {
				mMobilePrinter.disconnect();
				Toast.makeText(getApplicationContext(), "USB device removed", Toast.LENGTH_SHORT).show();
			}

		}
	};
}
