package com.zebra.mobileprintersample;

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.view.Menu;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioButton;

import com.zebra.printer.MobilePrinter;

public class MsrActivity extends Activity implements OnClickListener {
	private EditText mTrack1EditText;
	private EditText mTrack2EditText;
	private EditText mTrack3EditText;
	
	private byte[] mTrack1Data;
	private byte[] mTrack2Data;
	private byte[] mTrack3Data;
	
	private RadioButton mEnableRadioButton;
	private RadioButton mDisableRadioButton;
	private Button mSetButton;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_msr);
		
		Intent intent = getIntent();
		int extra = intent.getIntExtra(MainActivity.EXTRA_NAME_MSR_MODE, 0);
		EditText editText = (EditText) findViewById(R.id.editText1);
		
		switch (extra) {
		case MobilePrinter.MSR_MODE_TRACK123_COMMAND:
			editText.setText("Track 1/2/3 read mode command");
			break;
		case MobilePrinter.MSR_MODE_TRACK1_AUTO:
			editText.setText("Track 1 read mode auto trigger");
			break;
		case MobilePrinter.MSR_MODE_TRACK2_AUTO:
			editText.setText("Track 2 read mode auto trigger");
			break;
		case MobilePrinter.MSR_MODE_TRACK3_AUTO:
			editText.setText("Track 3 read mode auto trigger");
			break;
		case MobilePrinter.MSR_MODE_TRACK12_AUTO:
			editText.setText("Track 1/2 read mode auto trigger");
			break;
		case MobilePrinter.MSR_MODE_TRACK23_AUTO:
			editText.setText("Track 2/3 read mode auto trigger");
			break;
		case MobilePrinter.MSR_MODE_TRACK123_AUTO:
			editText.setText("Track 1/2/3 read mode auto trigger");
			break;
		case MobilePrinter.MSR_MODE_NOT_USED:
		default:
			editText.setText("MSR not used");
			break;
		}
		
		Button button1 = (Button) findViewById(R.id.button1);
		Button button2 = (Button) findViewById(R.id.button2);
		
		if (extra == MobilePrinter.MSR_MODE_TRACK123_COMMAND) {
			button1.setOnClickListener(this);
			button2.setOnClickListener(this);
		} else {
			button1.setEnabled(false);
			button2.setEnabled(false);
		}
		
		mTrack1EditText = (EditText) findViewById(R.id.editText2);
		mTrack2EditText = (EditText) findViewById(R.id.editText3);
		mTrack3EditText = (EditText) findViewById(R.id.editText4);
		
		mEnableRadioButton = (RadioButton) findViewById(R.id.radio0);
		mDisableRadioButton = (RadioButton) findViewById(R.id.radio1);
		
		Button button = (Button) findViewById(R.id.button3);
		button.setOnClickListener(this);
		mSetButton = (Button) findViewById(R.id.button4);
		
		button = (Button) findViewById(R.id.button5);
		button.setOnClickListener(this);
		button = (Button) findViewById(R.id.button6);
		button.setOnClickListener(this);
		
		IntentFilter filter = new IntentFilter();
		filter.addAction(MainActivity.ACTION_GET_MSR_TRACK_DATA);
		filter.addAction(MainActivity.ACTION_GET_MSR_ENCRYPTION);
		registerReceiver(mReceiver, filter);
	}
	
	@Override
	protected void onDestroy() {
		super.onDestroy();
		unregisterReceiver(mReceiver);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.activity_msr, menu);
		return true;
	}

	@Override
	public void onClick(View v) {
		switch (v.getId()) {
		case R.id.button1:
			MainActivity.mMobilePrinter.setMsrReaderMode();
			break;
			
		case R.id.button2:
			MainActivity.mMobilePrinter.cancelMsrReaderMode();
			break;
			
		case R.id.button3:
			MainActivity.mMobilePrinter.getMsrEncryption();
			break;
			
		case R.id.button4:
			MainActivity.mMobilePrinter.setMsrEncryption(mEnableRadioButton.isChecked());
			setResult(MainActivity.RESULT_CODE_MSR_MANAGER);
			finish();
			break;
			
		case R.id.button5:
			String trackData = mTrack1EditText.getText().toString();
			if (trackData.length() > 0) {
				byte[] decryptedData = MainActivity.mMobilePrinter.decryptTrackData(mTrack1Data);
				mTrack1EditText.setText(new String(decryptedData));
			}
			
			trackData = mTrack2EditText.getText().toString();
			if (trackData.length() > 0) {
				byte[] decryptedData = MainActivity.mMobilePrinter.decryptTrackData(mTrack2Data);
				mTrack2EditText.setText(new String(decryptedData));
			}
			
			trackData = mTrack3EditText.getText().toString();
			if (trackData.length() > 0) {
				byte[] decryptedData = MainActivity.mMobilePrinter.decryptTrackData(mTrack3Data);
				mTrack3EditText.setText(new String(decryptedData));
			}
			break;
			
		case R.id.button6:
			MainActivity.mMobilePrinter.clearMsrData();
			break;
		}
	}
	
	BroadcastReceiver mReceiver = new BroadcastReceiver() {

		@Override
		public void onReceive(Context context, Intent intent) {
			if (intent.getAction().equals(MainActivity.ACTION_GET_MSR_TRACK_DATA)) {
				Bundle bundle = intent.getBundleExtra(MainActivity.EXTRA_NAME_MSR_TRACK_DATA);
				
				mTrack1Data = bundle.getByteArray(MobilePrinter.MSR_TRACK1);
				if (mTrack1Data == null) {
					mTrack1EditText.setText("");
				} else {
					mTrack1EditText.setText(new String(mTrack1Data));
				}
				
				mTrack2Data = bundle.getByteArray(MobilePrinter.MSR_TRACK2);
				if (mTrack2Data == null) {
					mTrack2EditText.setText("");
				} else {
					mTrack2EditText.setText(new String(mTrack2Data));
				}
				
				mTrack3Data = bundle.getByteArray(MobilePrinter.MSR_TRACK3);
				if (mTrack3Data == null) {
					mTrack3EditText.setText("");
				} else {
					mTrack3EditText.setText(new String(mTrack3Data));
				}
			} else if (intent.getAction().equals(MainActivity.ACTION_GET_MSR_ENCRYPTION)) {
				mEnableRadioButton.setEnabled(true);
				mDisableRadioButton.setEnabled(true);
				mSetButton.setEnabled(true);
				mSetButton.setOnClickListener(MsrActivity.this);
				
				int encryption = intent.getIntExtra(MainActivity.EXTRA_NAME_MSR_ENCRYPTION, 0);
				if (encryption == 0) {
					mDisableRadioButton.setChecked(true);
				} else {
					mEnableRadioButton.setChecked(true);
				}
			}
		}
	};
}
